#pragma once
//  AppCommandHandler
//
//  This class provides the custom MQ command handling for the QMove application
//
//  Message Identifier:      QMV
//  Message Range:           0200 - 0299
//  Trace area assignment:   None
//  Version:                 1.4.2  Build: 37
//
//  Version History:
//
//  1.4.0		04/11/2010		Command Handler introduced
//  1.4.2		22/10/2011		VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010 - 2011
//

//  Runtime Includes
#include "../DXCommon/MTX/CommandHandler.h"

//  Application Includes
#include "AppTransactionHandler.h"

//  Custom Tell (MQ) Command strings
#define		MQ_CMD_STRING_RESTART		"RESTART"
#define		MQ_CMD_STRING_MAXTRANS		"MAXTRANS"

//  Encoded Custom Tell (MQ) commands
#define		MQ_COMMAND_RESTART			MQ_COMMAND_USER+1
#define		MQ_COMMAND_MAXTRANS			MQ_COMMAND_USER+2

//  Information Messages
#define		MSG_QMV0201I				"QMV0201I: %i transaction have been dispatched, %i completed, %i are running, max concurrency is %i."
#define		MSG_QMV0203I				"QMV0203I: Transaction processing has been suspended on queue '%s'."
#define		MSG_QMV0204I				"QMV0204I: Transaction processing has been resumed on queue '%s'."
#define		MSG_QMV0205I				"QMV0205I: Transaction processing is suspended on queue '%s'."
#define		MSG_QMV0207I				"QMV0207I: Maximum Concurrent Transactions on queue '%s' has been changed from %i to %i."
#define		MSG_QMV0208I				"QMV0208I: Transactions marked Completed: %i, Error: %i, Retried: %i, Delayed: %i."

//  Error Messages
#define		MSG_QMV0202E				"QMV0202E: No transaction queue is currently active for this processor."
#define		MSG_QMV0206E				"QMV0206E: Invalid setting for Max Concurrent Transactions on queue '%s' has been ignored."

//  Miscellaneous Definitions
#define		ADDIN_STATUS_RUNNING		"Running"
#define		ADDIN_STATUS_SUSPENDED		"Suspended"

class AppCommandHandler :
	public CommandHandler
{
public:
	//  Default constructor
	AppCommandHandler(void);

	//  Normal Constructor
	AppCommandHandler(ExecEnvironment *xeParent);

	//  Destructor
	~AppCommandHandler(void);

	//  Public Members
	TransactionQueue		*tqCurrent;								//  Pointer to transaction Queue
	AppTransactionHandler	*thCurrent;								//  Transaction Handler

protected:
	//  Protected Members

	//  Protected Functions
	WORD	virtual ParseCustomCommand(char *szCommand, char *szOptions, int iThreadID);	//  Parse a Custom Command
	WORD	virtual CustomCommandHandler(WORD wCommand, char *szOptions, int iThreadID);	//  Process Custom Commands
};
