#pragma once
//  AppRunSettings
//
//  This class provides the Application Execution settings for the QMove application
//
//  Message Identifier:      QMV
//  Message Range:           0001 - 0099
//  Trace area assignment:   None
//  Version:                 1.4.2  Build: 37
//
//  Version History:
//
//  1.0.0		- 14/11/2009		-	Initial Version
//  1.1.0		- 01/12/2009		-	Enhanced move operation modes supported
//  1.2.0		- 01/01/2010		-	Variable delay between transactions
//									-	Variable buffer size for copying
//  1.4.0		- 04/11/2010		-	Updated for 3.10.1 kernel & asynch transactions
//  1.4.2		- 22/10/2011		-	VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2009 - 2011
//

//  Application Includes
#include "../DXCommon/RunSettings.h"

//  Local Definitions

//  Local Constants
#define		MIN_DELAY				1							//  Minimum delay 1 sec
#define		MAX_DELAY				600							//  Maximum allowable delay 600s (10 mins)
#define		DEFAULT_IT_DELAY		120							//  Default delay 120s (2 mins)
#define		MIN_BFRK				4							//  Min transfer buffer 4k
#define		MAX_BFRK				1024						//  Max transfer buffer 1024k (1m)
#define		DEFAULT_BFRK			32							//  Default transfer buffer 32k
#define		MIN_THREADCOUNT			2							//  Minimum allowable threads
#define		MAX_THREADCOUNT			100							//  Maximum allowable threads
#define		DEFAULT_THREADCOUNT		5							//  Default threads
#define		DEFAULT_TXCOUNT			1							//  Default max 1 concurrent transaction
#define		DEFAULT_ARENASIZE		1							//  Default 1 Mb Arena Size

//  Error Messages
#define		MSG_QMV0001E		"QMV0001E: Too few parameters supplied."
#define		MSG_QMV0002E		"QMV0002E: Invalid program parameter '%s' has been ignored."
#define		MSG_QMV0003E		"QMV0003E: Invalid program switch '%s' has been ignored."
#define		MSG_QMV0004E		"QMV0004E: Invalid Transaction delay (-I:%i) supplied, program will not run."
#define		MSG_QMV0008E		"QMV0008E: Invalid Transfer Buffer Size (-B:%i) supplied, program will not run."
#define		MSG_QMV0010E		"QMV0010E: Invalid thread count %i supplied, program will not run."

//  Information Messages
#define		MSG_QMV0005I		"QMV0005I: Control database is '%s'."
#define		MSG_QMV0006I		"QMV0006I: Current logging level is: %s."
#define		MSG_QMV0007I		"QMV0007I: QMove will pause for %i seconds between transactions."
#define		MSG_QMV0009I		"QMV0009I: QMove will use a transfer buffer size of %iKb."
#define		MSG_QMV0011I		"QMV0011I: QMove will use %i threads."
#define		MSG_QMV0012I		"QMV0012I: QMove will allow a maximum of %i concurrent transactions."

//  Unnumbered messages

#define		MSG_USAGE1			"Usage:"
#define		MSG_USAGE2			"QMove ControlDb [-I:nn][-B:nn][-M:nn][-X:nn][-V][-T[:nn]][-D][-E]"
#define		MSG_USAGE3			"  ControlDb - Control database (relative to the Notes data directory)."
#define		MSG_USAGE4			"  [-I:nnn] - Delay between transactions 1-600 seconds."
#define		MSG_USAGE5			"  [-B:nnnnn] - Transfer buffer size in Kb (4-1024)."
#define		MSG_USAGE6			"  [-M:nn] - Use nn threads for execution."
#define		MSG_USAGE7			"  [-X:nn] - Max number of concurrent transactions."
#define		MSG_USAGE8			"  [-A:nnn] - Arena Size to use for copies in Mb."
#define		MSG_USAGE9			"  [-L] - Enable use of the Large File Copier."
#define		MSG_USAGE10			"  [-V]|[-T[:area]]|[-D] - Set logging level to Verbose, Trace or Debug."
#define		MSG_USAGE11			"  [-E] - Echo logging messages to the server console."
#define		MSG_USAGE12			"Copyright: Hadleigh Marshall Netherlands 2009 - 2011."

class AppRunSettings :
	public RunSettings
{
public:
	//  Constructor
	AppRunSettings(int argc, char * argv[]);

	//  Destructor
	~AppRunSettings(void);

	//  Public Members
	int		TransactionDelay;										//  Transaction Delay (in seconds)
	int		TransferBufferK;										//  Transfer Buffer size in Kb
	int		MaxCTX;													//  Max Concurrent transactions
	int		ThreadCount;											//  Number of threads to use
	int		ArenaSize;												//  Arena size to use for copies
	BOOL	UseLFC;													//  Use the Large File Copier

private:
	//  Private Functions
	BOOL SetDefaults(void);											//  Set the application run defaults
	BOOL ValidateParameters(int argc, char *argv[]);				//  Validate Parameters
	void ShowUsage(void);											//  Show program usage
	void ShowSettings(void);										//  Show the current settings
};
