#pragma once
//  AppTransactionHandler
//
//  This class provides the custom Transaction Queue handling for the QMove application
//
//  Message Identifier:      QMV
//  Message Range:           0300 - 0399
//  Trace area assignment:   None
//  Version:                 1.4.2  Build: 37
//
//  Version History:
//
//  1.4.0	-	04/11/2010		-	Transaction Handler introduced
//  1.4.1	-	21/12/2010		-	Switch to Runtime 3.10.3
// 									Fixed an issue with the transaction handler
//  1.4.2	-	22/10/2011		-	VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010 - 2011
//

//  Runtime Includes
#include "../DXCommon/MTX/TransactionHandler.h"						//  Transaction Queue Handler
#include "../DXCommon/DBC/MoveRequest.h"							//  High level move request

//  Additional API includes
#include <stats.h>

//  Application Includes
#include "AppRunSettings.h"											//  App Run Settings object
#include "QMover.h"													//  QMove engine 
#include "QMoveRequest.h"											//  QMove Request
#include "QMFeeder.h"												//  Feeder transaction processor

//  View Names
#define		VIEW_SERVERS		"Servers"

//  Engine processing states
#define		STATE_WAIT_FOR_WORK			1
#define		STATE_CHECK_AVAILABILITY	2
#define		STATE_EXEC_REQUEST			3
#define		STATE_HALTED				4
#define		STATE_REQUEUE_DELAYED		5
#define		STATE_PERFORM_SHUTDOWN		6
#define		STATE_TERMINAL_ERROR		7

//  Miscellaneous Definitions
#define		LOG_TRUNCATED_MSG	"....Transaction Log Truncated ......."
#define		CHAR_TAB			'\t'						//  Tab Character
#define		CHAR_LF				'\n'						//  Linefeed Character
#define		STATNAME_USERS		"Users"						//  Statistic name # users

//  Information Messages
#define		MSG_QMV0305I		"QMV0305I: Server '%s' shows %i current users against a limit of %i."
#define		MSG_QMV0321I		"QMV0321I: Transaction '%s' has been requeued to be retried."

//  Warning Messages
#define		MSG_QMV0312W		"QMV0312W: Server '%s' is unavailable, transaction '%s' will be delayed."

//  Error Messages
#define		MSG_QMV0301E		"QMV0301E: Unable to stamp the move start time in the request document."
#define		MSG_QMV0302E		"QMV0302E: Unable to stamp the move finish time in the request document."
#define		MSG_QMV0303E		"QMV0303E: Unable to populate a request from a document (Note ID: 0x%.4X)."
#define		MSG_QMV0304E		"QMV0304E: Unable to open transaction ID: '%s' (Note ID: 0x%.4X) in Queue '%s', status has not been updated."
#define		MSG_QMV0306E		"QMV0306E: Unable to allocate a list for the transaction history log, continuilg without one."
#define		MSG_QMV0307E		"QMV0307E: Unable to add a line to the transaction history log, the log is truncated."
#define		MSG_QMV0308E		"QMV0308E: Unable to add the transaction history log to the transaction document."
#define		MSG_QMV0309E		"QMV0309E: Unable to remove the old transaction history log from a document."
#define		MSG_QMV0310E		"QMV0310E: Unable to locate the Server Limits view."
#define		MSG_QMV0311E		"QMV0311E: Unable to open the Server Limits view."
#define		MSG_QMV0313E		"QMV0313E: Unable to get the name of the current server."
#define		MSG_QMV0314E		"QMV0314E: Unable to abbreviate the server name."
#define		MSG_QMV0315E		"QMV0315E: Unable to retrieve server statistic '%s'."
#define		MSG_QMV0316E		"QMV0316E: Unable to lookup server details for '%s'."
#define		MSG_QMV0317E		"QMV0317E: Unable to read entries in view/queue '%s' in the control database."
#define		MSG_QMV0318E		"QMV0318E: Unable to open Server document - '%s' (Note ID: '%lX')."
#define		MSG_QMV0319E		"QMV0319E: Unable to create a new transaction request document to retry transaction '%s'."
#define		MSG_QMV0320E		"QMV0320E: Unable to serialize data to transaction request document to retry transaction '%s'."
#define		MSG_QMV0322E		"QMV0322E: Transaction '%s' has exceeded the retry limit (%i) and will be failed."

class AppTransactionHandler :
	public TransactionHandler
{
public:
	//  Default constructor
	AppTransactionHandler(void);

	//  Normal Constructor
	AppTransactionHandler(MTExecutive *xeParent, int iThreadID);

	//  Destructor
	~AppTransactionHandler(void);

	//  Public Members
	QMover				*qmMaster;							//  Master Mover
	QMFeeder			*qfMaster;							//  Master Feeder

	//  Throughput statistics
	DWORD				dwCompleted;						//  Transactions marked completed
	DWORD				dwError;							//  Transaction marked error
	DWORD				dwRetried;							//  Transactions marked retried
	DWORD				dwDelayed;							//  Transactions marked delayed

private:
	//  Private Members

	//  Private Functions
	void LogTransactionMessage(char *szMsg, HANDLE hTXLog, int iThreadID);		//  Log a transaction message
	void AddHistory(HANDLE hTXLog, NOTEHANDLE hnTX, int iThreadID);				//  Add history to transaction document
	int	checkAvailability(QMoveRequest *mrCurrent, int iThreadID);				//  Check the availability of the source & target servers
	BOOL HasWildcards(char *szFileSpec);										//  Check a filespec for wild cards
protected:
	//  Protected Members

	//  Protected Functions
	void virtual SerializeTransaction(void *txObject, DBHANDLE hdbQueue, TransactionQueue *tqCurrent, int iThreadID);
	int virtual MarshallTransaction(NOTEID nidTX, NOTEHANDLE hnTX, TransactionQueue *tqCurrent, void **txObject, void **xxObject, UINT *Attrs, int *Priority, int iThreadID);
};

