//  QFeedRequest
//
//  This class extends the standard DXRequest class to carry additional information needed by the
//  QCFeeder processing exit.
//
//  Message Identifier:      N/A
//  Message Range:           0000 - 0000
//  Trace area assignment:   None
//  Version:                 1.4.2  Build: 37
//
//  Version History:
//
//  1.4.0		04/11/2010   -  Asynch transcation queue handler added.
// 								Incorporated RunTime (3.10.1) changes.
//  1.4.2		22/10/2011	 -	VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010 - 2011
//

//  Application Includes
#include "QFeedRequest.h"

//  Constructor
QFeedRequest::QFeedRequest()
{
	hTXLog = NULLHANDLE;
	dwTransactionsCreated = 0;
	qmrFeeder = NULL;
}

//  Destructor
QFeedRequest::~QFeedRequest()
{
}

//  InheritFrom
//
//  This function will populate the basic members of the current request from a passed request
//  Also populating the items in the extended request
//
//  Parameters
//
//  1  -  DXRequest *		-  Pointer to a DXRequest object that defines the eXplorer search scope and constraints
//
//  Returns
//
//  API Use Notes:
//
//
void QFeedRequest::InheritFrom(DXRequest *dxrParent)
{
	QFeedRequest			*qfrParent;							//  Parent Feeder Request

	//  Cast the passed request 
	qfrParent = (QFeedRequest *) dxrParent;

	//  Copy the specific items from the parent
	hTXLog = qfrParent->hTXLog;
	qmrFeeder = qfrParent->qmrFeeder;
	return BaseInheritFrom(dxrParent);
}

//  CreateNewInstance
//
//  This function will create a new DXRequest instance and return it to the caller. 
//  This function can be overridden in extending classes, to allow extending classes to be handled by
//  the DX core functions.
//
//  Parameters
//
//  Returns
//
//  API Use Notes:
//
//
DXRequest * QFeedRequest::CreateNewInstance(void)
{
	return new QFeedRequest();	
}

//  PromoteScores
//
//  This function will accumulate the scanning scores from this request into a higher level request
//
//  Parameters
//
//  1  -  DXRequest *		-  Pointer to a DXRequest object that defines the eXplorer search scope and constraints
//
//  Returns
//
//  API Use Notes:
//
//  In any replacement invoke the BasePromoteScores() protected method to perform existing promotion
//
void QFeedRequest::PromoteScores(DXRequest *dxrParent)
{
	QFeedRequest			*qfrParent;							//  Parent Queue Feed Request

	//  Cast the passed request 
	qfrParent = (QFeedRequest *) dxrParent;

	qfrParent->dwTransactionsCreated = qfrParent->dwTransactionsCreated + dwTransactionsCreated;
	return BasePromoteScores(dxrParent);
}
