#pragma once
//  QCFeedProcessor
//
//  This class provides the database level exit for Domino eXplorer for processing individual
//  QFeedRequest objects.
//
//  Message Identifier:      QMV
//  Message Range:           0600 - 0699
//  Trace area assignment:   None
//  Version:                 1.4.0  Build: 17
//
//  Version History:
//
//  1.4.0		04/11/2010   -  Asynch transcation queue handler added.
// 								Incorporated RunTime (3.10.1) changes.
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010 - 2011
//

//  Platform Includes
#include "../DXCommon/Platform/PlatBase.h"

//  Runtime Includes
#include "../DXCommon/MTDX/DXReporter.h"					//  Reporter interface

//  Application Includes
#include "QFeedRequest.h"

//  Trace Messages
#define		MSG_QMV0606T		"QMV0606T: Bulding move request for Dir: '%s' Db: '%s'."

//  Information Messages
#define		MSG_QMV0605I		"QMV0605I: QMove transaction '%s' has been built for source '%s'."

//  Error Messages
#define		MSG_QMV0601E		"QMV0601E: Unable to add a line to the transaction history log, the log is truncated."
#define		MSG_QMV0602E		"QMV0602E: Unable to configure a request for '%s\\%s', no tranaction created."
#define		MSG_QMV0603E		"QMV0603E: Unable to create a new transaction note for '%s\\%s'."
#define		MSG_QMV0604E		"QMV0604E: Unable to serialize a new transaction for '%s\\%s'."

//  Local Definitions
#define		REPORTER_DESC		"QMove transaction feeder"
#define		HISTORY_MUTEX		APP_MUTEX_1										//  Define the history Mutex
#define		LOG_TRUNCATED_MSG	"....Transaction Log Truncated ......."

class QMFeedProcessor :
	public DXReporter
{
public:
	//  Default constructor
	QMFeedProcessor(void);

	//  Normal Constructor
	QMFeedProcessor(MTExecutive *xeParent, int iThreadID);

	//  Destructor
	~QMFeedProcessor(void);

	//  Public Members

	//  Public Functions
	virtual BOOL ReportOnThisDb(DXRequest *dxrRQ, DBHANDLE hdbEntity, int iThreadID);	//  Report on the selected Db
	virtual void GetDescription(char *szDescription);									//  Return the description of this reporter

private:
	//  Private Members
	MTExecutive		*xeLocal;															//  Runtime
	int				ParentThread;														//  Owning Thread

	//  Private Functions
	void LogTransactionMessage(char *szMsg, HANDLE hTXLog, int iThreadID);				//  Log a transaction message
	BOOL PopulateRequest(QMoveRequest *qmrRQ, QFeedRequest *qfrCurrent, int iThreadID);  //  Populate a new request
};
