#pragma once
//  QMFeeder
//
//  This class provides the capability of handling "Feeder" transactions within QMove.
//  Feeder transaction are those that specify a higher level of source/target than database level and
//  therefore result in scanning to generate database level transactions.
//
//  Message Identifier:      QMV
//  Message Range:           0400 - 0499
//  Trace area assignment:   None
//  Version:                 1.4.2  Build: 37
//
//  Version History:
//
//  1.4.0		17/11/2010   -  Asynch transcation queue handler added.
// 								Incorporated RunTime (3.10.1) changes.
//  1.4.2		22/10/2011	 -	VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010 - 2011
//

//  Platform Includes
#include "../DXCommon/Platform/PlatBase.h"

//  Runtime Includes
#include "../DXCommon/MTX/MTExecutive.h"					//  Core Runtime
#include "../DXCommon/MTDX/DominoExplorer.h"				//  Domino Explorer (DX)
#include "../DXCommon/MTDX/DXReporter.h"					//  Reporter interface
#include "../DXCommon/Threads/Runnable.h"					//  Runnable Interface
#include "../DXCommon/DBC/PartCopyRequest.h"				//  Low level copy request

//  Application Includes
#include "QFeedRequest.h"
#include "QMFeedProcessor.h"

//  Information Messages
#define		MSG_QMV0405I		"QMV0405I: Feeder transaction has completed and created %i QMove transactions."

//  Error Messages
#define		MSG_QMV0401E		"QMV0401E: Feeder transaction engine failed to initialize, transaction cannot be processed."
#define		MSG_QMV0402E		"QMV0402E: Unable to parse the source server/database combination to create a valid search."
#define		MSG_QMV0403E		"QMV0403E: Unable to dispatch the Domino eXplorer request, transaction not executed."
#define		MSG_QMV0404E		"QMV0404E: Unable to add a line to the transaction history log, the log is truncated."

//  Local Definitions
#define		HISTORY_MUTEX		APP_MUTEX_1										//  Define the history Mutex
#define		LOG_TRUNCATED_MSG	"....Transaction Log Truncated ......."

class QMFeeder :
	public Runnable
{
public:
	//  Default constructor
	QMFeeder(void);

	//  Normal Constructor
	QMFeeder(MTExecutive *xeParent, int iThreadID);

	//  Destructor
	~QMFeeder(void);

	//  Public Members

	//  Public Functions
	void virtual ExecuteThisRequest(void *pReqObject, int iThreadID);			//  Multi thread request entry point

private:
	//  Private Members
	MTExecutive		*xeLocal;													//  Runtime
	DominoExplorer	*dxLocal;													//  Domino Explorer
	QMFeedProcessor *qpLocal;													//  Feed Processor
	int				ParentThread;												//  Owning Thread
	BOOL			bOnDemandInitDone;											//  Initialisation done

	//  Private Functions
	void ProcessThisRequest(QMoveRequest *qmrRQ, int iThreadID);				//  Process a Feeder Request
	BOOL DoLazyInitialisation(int iThreadID);									//  Perform on-demend initialisation
	void LogTransactionMessage(char *szMsg, HANDLE hTXLog, int iThreadID);		//  Log a transaction message
};




