//  QMoveRequest
//
//  This class extends the DX MoveRequest class to support the needs of the QMover Class.
//
//  Version:                 1.4.2  Build: 37
//
//  Version History:
//
//	1.0.0 - 01/11/2009		-	Initial Version
//  1.1.0 - 01/12/2009		-	Enhanced move operation modes supported
//  1.4.0 - 15/11/2010		-	MTIO Support added
//  1.4.2 - 22/10/2011		-	VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2009 - 2011
//

//  Application Includes
#include "QMoveRequest.h"

//  Constructor
QMoveRequest::QMoveRequest(void)
{
}

//  Destructor
QMoveRequest::~QMoveRequest(void)
{
}

//  populate
//
//  This function will populate the request transaction data from the request note.
//
//  Parameters
//
//  1  -  NOTEID		-  NoteID of the request note
//  2  -  NOTEHANDLE	-  Handle of the request note
//
//  Returns
//
//  BOOL  -				true if the request was loaded
//						false if the request could not be loaded
//
//  API Use Notes:
//
//
BOOL QMoveRequest::populate(NOTEID nidRQ, NOTEHANDLE hnRQ)
{
	WORD				wRetLen;					//  Returned length
	char				szItemValue[MAXPATH + 1];	//  Generic item value
	char				szMoveOps[MAX_OPS];			//  Move operations
	char				*pOps;						//  Operations pointer

	nidRequest = nidRQ;

	wRetLen = NSFItemGetText(hnRQ, TRANSDOC_TRANSREQID, szTransID, MAXWORD);
	szTransID[wRetLen] = '\0';

	wRetLen = NSFItemGetText(hnRQ, TRANSDOC_SOURCEDBPATH, szSourceDatabase, MAXWORD);
	szSourceDatabase[wRetLen] = '\0';

	wRetLen = NSFItemGetText(hnRQ, TRANSDOC_TARGETDIR, szTargetDir, MAXWORD);
	szTargetDir[wRetLen] = '\0';

	wRetLen = NSFItemGetText(hnRQ, TRANSDOC_MOVEOPS, szMoveOps, MAXWORD);
	szMoveOps[wRetLen] = '\0';

	if (wRetLen > 0)
	{
		//  Decode the move operations and set them 
		PerformCopy = FALSE;
		PerformDelete = FALSE;
		PerformLink = FALSE;

		pOps = szMoveOps;
		while (*pOps != '\0')
		{
			if (*pOps == 'C') PerformCopy = TRUE;					//  Copy operation is required
			if (*pOps == 'D') PerformDelete = TRUE;					//  Delete operation is required
			if (*pOps == 'L') PerformLink = TRUE;					//  Link operation is required
			pOps++;
		}
	}
	else
	{
		//  For backwards compatability set all ops if the encoded string is not available
		PerformCopy = TRUE;
		PerformDelete = TRUE;
		PerformLink = TRUE;
	}

	wRetLen = NSFItemGetText(hnRQ, TRANSDOC_ALLOW_RETRY, szItemValue, MAXWORD);
	szItemValue[wRetLen] = '\0';
	if (strcmp(szItemValue, "0") != 0)
	{
		AllowRetry = TRUE;
	}
	else
	{
		AllowRetry = FALSE;
	}

	wRetLen = NSFItemGetText(hnRQ, TRANSDOC_RETRY_COUNT, szItemValue, MAXWORD);
	szItemValue[wRetLen] = '\0';
	RetryCount = atoi(szItemValue);

	wRetLen = NSFItemGetText(hnRQ, TRANSDOC_RETRY_LIMIT, szItemValue, MAXWORD);
	szItemValue[wRetLen] = '\0';
	RetryLimit = atoi(szItemValue);

	wRetLen = NSFItemGetText(hnRQ, TRANSDOC_RESTART_ACTIONS, szItemValue, MAXWORD);
	szItemValue[wRetLen] = '\0';
	RestartActions = atoi(szItemValue);

	wRetLen = NSFItemGetText(hnRQ, TRANSDOC_FAULT_TOLERANCE, szItemValue, MAXWORD);
	szItemValue[wRetLen] = '\0';
	FaultTolerance = atoi(szItemValue);

	//  Check default/conflict settings
	if (AllowRetry)
	{
		if (RetryLimit == 0) RetryLimit = DEFAULT_RETRY_LIMIT;
	}

	return TRUE;
}

//  writeStats
//
//  This function will write the accumulated stats for this request back to the request document
//
//  Parameters
//
//  1  -  NOTEHANDLE	-  Handle of the request note
//
//  Returns
//
//  BOOL  -				TRUE if the stats were written
//						FALSE if the stats could not be written
//
//  API Use Notes:
//
//
BOOL QMoveRequest::writeStats(HANDLE hnRQ)
{
	STATUS			stAPIRC;								//  Return code from Notes API Calls
	int				iFails = 0;								//  Count of write failures
	NUMBER			nGeneric;								//  Generic Number

	nGeneric = dwDataDocsCopied;
	stAPIRC = NSFItemSetNumber(hnRQ, TRANSDOC_DATA_COPIED, &nGeneric);
	if (ERR(stAPIRC) != NOERROR) iFails++;
	nGeneric = dwTotalElapsed;
	stAPIRC = NSFItemSetNumber(hnRQ, TRANSDOC_ELAPSED, &nGeneric);
	if (ERR(stAPIRC) != NOERROR) iFails++;
	nGeneric = dwSizeKB;
	stAPIRC = NSFItemSetNumber(hnRQ, TRANSDOC_SRCDATA_SIZE, &nGeneric);
	if (ERR(stAPIRC) != NOERROR) iFails++;

	//  Make sure we don't raise divide by zero
	if (dwTotalElapsed == 0) dwTotalElapsed = 1;

	//  Compute and write the throughput rates
	nGeneric = dwDataDocsCopied / dwTotalElapsed;
	stAPIRC = NSFItemSetNumber(hnRQ, TRANSDOC_THROUGHPUT_DOCS, &nGeneric);
	if (ERR(stAPIRC) != NOERROR) iFails++;

	nGeneric = dwSizeKB / dwTotalElapsed;
	stAPIRC = NSFItemSetNumber(hnRQ, TRANSDOC_THROUGHPUT_KBSEC, &nGeneric);
	if (ERR(stAPIRC) != NOERROR) iFails++;

	if (iFails > 0) return FALSE;
	return TRUE;
}

//  serialize
//
//  This function will serialize the current request data to a Request Transaction document
//
//  Parameters
//
//  Returns
//
//  BOOL  -				TRUE if the clone was written
//						FALSE if the clone could not be written
//
//  API Use Notes:
//
//
BOOL QMoveRequest::serialize(NOTEID nidRQ, NOTEHANDLE hnRQ, ExecEnvironment *xeLocal)
{
	STATUS		stAPIRC;								//  return code from Notes API calls
	char		szItemValue[MAXPATH + 1];				//  Generic item value
	char		szMsg[MAX_MSG + 1];						//  Generic Message Buffer

	nidRequest = nidRQ;

	//  Populate the transaction document

	stAPIRC = NSFItemSetText(hnRQ, "Form", TRANSDOC_FORM_NAME, MAXWORD);
	if (ERR(stAPIRC) != NOERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_QMV1001E, "Form");
		xeLocal->LogMessage(szMsg);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}

	stAPIRC = NSFItemSetText(hnRQ, TRANSDOC_TRANSREQID, szTransID, MAXWORD);
	if (ERR(stAPIRC) != NOERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_QMV1001E, TRANSDOC_TRANSREQID);
		xeLocal->LogMessage(szMsg);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}

	stAPIRC = NSFItemSetText(hnRQ, TRANSDOC_SOURCEDBPATH, szSourceDatabase, MAXWORD);
	if (ERR(stAPIRC) != NOERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_QMV1001E, TRANSDOC_SOURCEDBPATH);
		xeLocal->LogMessage(szMsg);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}

	stAPIRC = NSFItemSetText(hnRQ, TRANSDOC_TARGETDIR, szTargetDir, MAXWORD);
	if (ERR(stAPIRC) != NOERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_QMV1001E, TRANSDOC_TARGETDIR);
		xeLocal->LogMessage(szMsg);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}

	//  Compile the move operations
	szItemValue[0] = '\0';
	if (PerformCopy) strcat_s(szItemValue, MAXPATH, "C");
	if (PerformDelete) strcat_s(szItemValue, MAXPATH, "D");
	if (PerformLink) strcat_s(szItemValue, MAXPATH, "L");

	stAPIRC = NSFItemSetText(hnRQ, TRANSDOC_MOVEOPS, szItemValue, MAXWORD);
	if (ERR(stAPIRC) != NOERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_QMV1001E, TRANSDOC_MOVEOPS);
		xeLocal->LogMessage(szMsg);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}

	strcpy_s(szItemValue, MAXPATH, "0");
	if (AllowRetry) strcpy_s(szItemValue, MAXPATH, "1");

	stAPIRC = NSFItemSetText(hnRQ, TRANSDOC_ALLOW_RETRY, szItemValue, MAXWORD);
	if (ERR(stAPIRC) != NOERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_QMV1001E, TRANSDOC_ALLOW_RETRY);
		xeLocal->LogMessage(szMsg);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}

	sprintf_s(szItemValue, MAXPATH, "%i", RetryCount);

	stAPIRC = NSFItemSetText(hnRQ, TRANSDOC_RETRY_COUNT, szItemValue, MAXWORD);
	if (ERR(stAPIRC) != NOERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_QMV1001E, TRANSDOC_RETRY_COUNT);
		xeLocal->LogMessage(szMsg);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}

	sprintf_s(szItemValue, MAXPATH, "%i", RetryLimit);

	stAPIRC = NSFItemSetText(hnRQ, TRANSDOC_RETRY_LIMIT, szItemValue, MAXWORD);
	if (ERR(stAPIRC) != NOERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_QMV1001E, TRANSDOC_RETRY_LIMIT);
		xeLocal->LogMessage(szMsg);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}

	sprintf_s(szItemValue, MAXPATH, "%i", RestartActions);

	stAPIRC = NSFItemSetText(hnRQ, TRANSDOC_RESTART_ACTIONS, szItemValue, MAXWORD);
	if (ERR(stAPIRC) != NOERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_QMV1001E, TRANSDOC_RESTART_ACTIONS);
		xeLocal->LogMessage(szMsg);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}

	sprintf_s(szItemValue, MAXPATH, "%i", FaultTolerance);

	stAPIRC = NSFItemSetText(hnRQ, TRANSDOC_FAULT_TOLERANCE, szItemValue, MAXWORD);
	if (ERR(stAPIRC) != NOERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_QMV1001E, TRANSDOC_FAULT_TOLERANCE);
		xeLocal->LogMessage(szMsg);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}


	//  Update the note
	stAPIRC = NSFNoteUpdate(hnRQ, UPDATE_FORCE);
	if (ERR(stAPIRC) != NOERROR)
	{
		xeLocal->LogMessage(MSG_QMV1002E);
		xeLocal->GetAPIMessage(ERR(stAPIRC), szMsg);
		xeLocal->LogMessage(szMsg);
		return(FALSE);
	}

	return TRUE;
}

