#pragma once
//  QMoveRequest
//
//  This class extends the DX MoveRequest class to support the needs of the QMover Class.
//
//  Version:                 1.4.2  Build: 37
//
//  Version History:
//
//	1.0.0 - 01/11/2009		-	Initial Version
//  1.1.0 - 01/12/2009		-	Enhanced move operation modes supported
//  1.4.0 - 15/11/2010		-	MTIO Support added
//  1.4.2 - 22/10/2011		-	VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2009 - 2011
//

//  Platform Includes
#include "../DXCommon/Platform/PlatBase.h"
#include "../DXCommon/MTX/MTExecutive.h"

//  Application Includes
#include "../DXCommon/DBC/MoveRequest.h"

//  Request Document Item Names
#define 	TRANSDOC_STATUS				"Status"
#define 	TRANSDOC_TRANSREQID			"TransReqID"
#define		TRANSDOC_SOURCESERVER		"SourceServer"
#define		TRANSDOC_SOURCEDBPATH		"SourceDatabase"
#define		TRANSDOC_TARGETDIR			"TargetDirectory"
#define		TRANSDOC_STARTTIME			"StartCopyTime"
#define		TRANSDOC_FINISHTIME			"FinishCopyTime"
#define		TRANSDOC_MOVEOPS			"OperationMode"
#define		TRANSDOC_HISTORY			"History"
#define		TRANSDOC_ELAPSED			"fcopy_Elapsed"
#define		TRANSDOC_THROUGHPUT_DOCS	"fcopy_DocsSecThroughput"
#define		TRANSDOC_THROUGHPUT_KBSEC	"fcopy_KbSecThroughput"
#define		SERVERDOC_SRCLIMIT			"SourceUserLimit"
#define		SERVERDOC_TARGLIMIT			"TargetUserLimit"
#define		TRANSDOC_DATA_ELAPSED		"fcopy_Data_Elapsed"
#define		TRANSDOC_DESIGN_ELAPSED		"fcopy_Design_Elapsed"
#define		TRANSDOC_DATA_COPIED		"fcopy_Data_Docs"
#define		TRANSDOC_DESIGN_COPIED		"fcopy_Design_Docs"
#define		TRANSDOC_FOLDERS_COPIED		"fcopy_Folder_Docs"
#define		TRANSDOC_FOLDERDOCS_COPIED	"fcopy_In_Folder_Docs"
#define		TRANSDOC_ACCESSERRORS		"AccessErrors"
#define		TRANSDOC_SRCDATA_COUNT		"fcopy_HSrc_Docs"
#define		TRANSDOC_SRCDATA_SIZE		"fcopy_HSrc_KBs"
#define		TRANSDOC_ALLOW_RETRY		"AllowRetry"
#define		TRANSDOC_RETRY_COUNT		"RetryCount"
#define		TRANSDOC_RETRY_LIMIT		"Retrylimit"
#define		TRANSDOC_RESTART_ACTIONS	"RestartActions"
#define		TRANSDOC_FAULT_TOLERANCE	"FaultTolerance"
#define		TRANSDOC_FORM_NAME			"QMTX"
#define		TRANSDOC_URGENT_FLAG		"Urgent"
#define		TRANSDOC_APPROVED_FLAG		"Approved"

//  Request Status Field Values
#define		TRANSDOC_STATUS_NEW			"NEW"
#define		TRANSDOC_STATUS_INPROGRESS	"INPROGRESS"
#define		TRANSDOC_STATUS_COMPLETED	"COMPLETED"
#define		TRANSDOC_STATUS_ERROR		"ERROR"
#define		TRANSDOC_STATUS_DELAYED		"DELAYED"

//  Error Messages
#define		MSG_QMV1001E				"QMV1001E: Unable to write %s field onto a cloned request transaction."
#define		MSG_QMV1002E				"QMV1002E: Unable to update a cloned request transaction."

//  Miscellaneous definitions
#define		BYTES_PER_KB				1024
#define		MAX_OPS						32							//  Buffer space for encoded operations
#define		DEFAULT_RETRY_LIMIT			3							//  Default Retry limit

class QMoveRequest :
	public MoveRequest
{
public:
	//  Constructor
	QMoveRequest(void);

	//  Destructor
	~QMoveRequest(void);

	//  Public Members
	NOTEID		nidRequest;												//  NoteID of the request
	HANDLE		hTXLog;													//  Handle to the transaction log

	//  Public Functions
	BOOL populate(NOTEID nidRQ, NOTEHANDLE hnRQ);						//  Populate a request from a note
	BOOL writeStats(NOTEHANDLE hnRQ);									//  Write the stats back to the request
	BOOL serialize(NOTEID nidRQ, NOTEHANDLE hnRQ, ExecEnvironment *xeLocal);	//  Clone this request for an error retry
};
